/**
 * Webアプリケーション用のツール類.
 *
 * Version 0.9.0.0
 *
 * このプログラムのライセンスについて。
 * このプログラムは、MITライセンス(The MIT License)の元で公開します。
 * Released under the MIT Licenses.
 *
 * ライセンスの原文は、以下のURLを参照してください。
 * http://www.yscjp.com/licenses/MIT_LICENSE.txt
 *
 * Copyright (c) 2010-2012 Yagyu Teru, http://yscjp.com/
 */
var com;
if (!com) com = {};
if (!com.yscjp) com.yscjp = {};
if (!com.yscjp.jsapp) com.yscjp.jsapp = {};
if (!com.yscjp.jsapp.element) com.yscjp.jsapp.element = {};

/**
 *
 * @class 要素を移動やサイズの変更をさせる際の、位置や移動／変更量の管理を行います.
 */
com.yscjp.jsapp.element.GridManager = function () {};
(function () {

    var ArgChk = com.yscjp.jsapp.util.ArgChk;
    var GridManager = com.yscjp.jsapp.element.GridManager;
    var HtmlPoint = com.yscjp.jsapp.tools.HtmlPoint;
    var Log = com.yscjp.jsapp.util.Log;

    /**
     * 初期化を行います.
     * @param gridWidthX X方向のグリッド幅です。0を指定した場合、移動／変更量の管理は行いません。
     * @param gridWidthY Y方向のグリッド幅です。0を指定した場合、移動／変更量の管理は行いません。
     * @param absorptionRangeX X方向のグリッドへの吸着量です。
     * @param absorptionRangeY Y方向のグリッドへの吸着量です。
     * @param referencePoint グリッドの基準点です。
     * @memberOf com.yscjp.jsapp.element.GridManager
     */
    function init(gridWidthX, gridWidthY, absorptionRangeX, absorptionRangeY, referencePoint) {

        ArgChk.typeOf(gridWidthX, "number");
        ArgChk.typeOf(gridWidthY, "number");
        ArgChk.typeOf(absorptionRangeX, "number");
        ArgChk.typeOf(absorptionRangeY, "number");
        ArgChk.instanceOf(referencePoint, HtmlPoint);

        this.gridWidthX = gridWidthX;
        this.gridWidthY = gridWidthY;
        this.absorptionRangeX = absorptionRangeX;
        this.absorptionRangeY = absorptionRangeY;
        this.referencePoint = referencePoint;
    }


    /**
     * 上方向へ移動した際の位置を返します.
     * 移動させる必要がある場合は、移動させるグリッドのyを返します。
     * 移動させる必要が無い場合は、-1を返します。
     * Y方向のグリッド幅が0の場合は、current.yを返します。
     * @param current 現在位置
     * @param brfore 前の位置
     * @returns 移動させるグリッドのy、移動させる必要が無い場合は-1
     * @memberOf com.yscjp.jsapp.element.GridManager
     */
    function goUpper(current, brfore) {

        return this.goUL(current, brfore);
    }


    /**
     * 下方向へ移動した際の位置を返します.
     * 移動させる必要がある場合は、移動させるグリッドのyを返します。
     * 移動させる必要が無い場合は、-1を返します。
     * Y方向のグリッド幅が0の場合は、current.yを返します。
     * @param current 現在位置
     * @param brfore 前の位置
     * @returns 移動させるグリッドのy、移動させる必要が無い場合は-1
     * @memberOf com.yscjp.jsapp.element.GridManager
     */
    function goLower(current, brfore) {

        return this.goUL(current, brfore);
    }


    /**
     * 上下方向へ移動した際の位置を返します.
     * 移動させる必要がある場合は、移動させるグリッドのyを返します。
     * 移動させる必要が無い場合は、-1を返します。
     * Y方向のグリッド幅が0の場合は、current.yを返します。
     * @param current 現在位置
     * @param brfore 前の位置
     * @returns 移動させるグリッドのy、移動させる必要が無い場合は-1
     * @memberOf com.yscjp.jsapp.element.GridManager
     */
    function goUL(current, brfore) {

        if (0 == this.gridWidthY) {

            return current.y;
        }

        ArgChk.instanceOf(current, HtmlPoint);
        brfore = ArgChk.instanceOf(brfore, HtmlPoint, null);

        var near = this.nearGrid(current);
        if (this.isMove(current.y, near.y, this.absorptionRangeY)) {

            return near.y;
        }

        var far = farLine(current.y, brfore.y, this.gridWidthY, this.referencePoint.y);

        return far == -1 ? brfore.y : far;
    }


    /**
     * 左方向へ移動した際の位置を返します.
     * 移動させる必要がある場合は、移動させるグリッドのxを返します。
     * 移動させる必要が無い場合は、-1を返します。
     * X方向のグリッド幅が0の場合は、current.xを返します。
     * @param current 現在位置
     * @param brfore 前の位置
     * @returns 移動させるグリッドのx、移動させる必要が無い場合は-1
     * @memberOf com.yscjp.jsapp.element.GridManager
     */
    function goLeft(current, brfore) {

        return this.goLR(current, brfore);
    }


    /**
     * 右方向へ移動した際の位置を返します.
     * 移動させる必要がある場合は、移動させるグリッドのxを返します。
     * 移動させる必要が無い場合は、-1を返します。
     * X方向のグリッド幅が0の場合は、current.xを返します。
     * @param current 現在位置
     * @param brfore 前の位置
     * @returns 移動させるグリッドのx、移動させる必要が無い場合は-1
     * @memberOf com.yscjp.jsapp.element.GridManager
     */
    function goRight(current, brfore) {

        return this.goLR(current, brfore);
    }


    /**
     * 左右方向へ移動した際の位置を返します.
     * 移動させる必要がある場合は、移動させるグリッドのxを返します。
     * 移動させる必要が無い場合は、-1を返します。
     * X方向のグリッド幅が0の場合は、current.xを返します。
     * @param current 現在位置
     * @param brfore 前の位置
     * @returns 移動させるグリッドのx、移動させる必要が無い場合は-1
     * @memberOf com.yscjp.jsapp.element.GridManager
     */
    function goLR(current, brfore) {

        if (0 == this.gridWidthX) {

            return current.x;
        }

        ArgChk.instanceOf(current, HtmlPoint);
        brfore = ArgChk.instanceOf(brfore, HtmlPoint, null);

        var near = this.nearGrid(current);
        if (this.isMove(current.x, near.x, this.absorptionRangeX)) {

            return near.x;
        }

        var far = farLine(current.x, brfore.x, this.gridWidthX, this.referencePoint.x);

        return far == -1 ? brfore.x : far;
    }

    function goLRUL(current, brfore) {

        var x = this.goLR(current, brfore);
        var y = this.goUL(current, brfore);
        var point = new HtmlPoint(x == -1 ? brfore.x : x, y == -1 ? brfore.y : y);
        return point;
    }

    function goUpperLeft(current, brfore) {

        var x = this.goLeft(current, brfore);
        var y = this.goUpper(current, brfore);
        var point = new HtmlPoint(x == -1 ? brfore.x : x, y == -1 ? brfore.y : y);
        return point;
    }

    function goUpperRight(current, brfore) {

        var x = this.goRight(current, brfore);
        var y = this.goUpper(current, brfore);
        var point = new HtmlPoint(x == -1 ? brfore.x : x, y == -1 ? brfore.y : y);
        return point;
    }

    function goLowerLeft(current, brfore) {

        var x = this.goLeft(current, brfore);
        var y = this.goLower(current, brfore);
        var point = new HtmlPoint(x == -1 ? brfore.x : x, y == -1 ? brfore.y : y);
        return point;
    }

    function goLowerRight(current, brfore) {

        var x = this.goRight(current, brfore);
        var y = this.goLower(current, brfore);
        var point = new HtmlPoint(x == -1 ? brfore.x : x, y == -1 ? brfore.y : y);
        return point;
    }

    function isMove(currentLine, nearLine, absorptionRange) {

        if (absorptionRange < Math.abs(nearLine - currentLine)) {

            /*
             * 吸着範囲外なので、移動なし。
             */
            return false;
        }

        return true;
    }

    function farLine(currentLine, brforeLine, gridWidth, referenceLine) {

        var distance = currentLine - referenceLine;
        var index = distance / gridWidth;

        var brforeDistance = brforeLine - referenceLine;
        var brforeIndex = brforeDistance / gridWidth;

        if (Math.floor(index) == Math.floor(brforeIndex)) {

            /*
             * currentLineとbrforeLineが、同じグリッド内にある場合は、-1を返す。
             */
            return -1;
        }

        var line;

        if (currentLine < brforeLine) {

            /*
             * 座標系として、上または左方向に移動した。
             * この場合、currentLineより大きくて一番近いグリッドラインを探す。
             */
            line =  Math.ceil(index) * gridWidth + referenceLine;   // 結果を切り上げ、大きい方のグリッド線を取得する

        } else {

            /*
             * 座標系として、下または右方向に移動した。
             * この場合、currentLineより小さいくて一番近いグリッドラインを探す。
             */
            line =  Math.floor(index) * gridWidth + referenceLine;  // 結果を切り捨て、小さい方のグリッド線を取得する
        }

        return line;
    }

    function nearGrid(current) {

        var gridPointX = nearLine(current.x, this.gridWidthX, this.referencePoint.x);
        var gridPointY = nearLine(current.y, this.gridWidthY, this.referencePoint.y);

        return new HtmlPoint(gridPointX, gridPointY);
    }

    function nextGrid(current, direction) {

        var x;
        var y;

        switch (direction) {

            case "N" : // 北（上）

                x = current.x;
                y = nearLine(current.y - (this.gridWidthY / 2 + 1), this.gridWidthY, this.referencePoint.y);
                break;

            case "E" : // 東（右）

                x = nearLine(current.x + (this.gridWidthX / 2 + 1), this.gridWidthX, this.referencePoint.x);
                y = current.y;
                break;

            case "S" : // 南（下）

                x = current.x;
                y = nearLine(current.y + (this.gridWidthY / 2 + 1), this.gridWidthY, this.referencePoint.y);
                break;

            case "W" : // 西（左）

                x = nearLine(current.x - (this.gridWidthX / 2 + 1), this.gridWidthX, this.referencePoint.x);
                y = current.y;
                break;

            default :

                /* 引数不正なので、移動しない */
                return current;
        }

        return new HtmlPoint(x, y);
    }

    function nearLine(currentLine, gridWidth, referenceLine) {

        var distance = currentLine - referenceLine;
        var index = distance / gridWidth;
        var gridLine = Math.floor(index) * gridWidth + referenceLine;  // 結果を切り捨て、小さい方のグリッド線を取得する
        if ((gridWidth / 2) <= (currentLine - gridLine)) {

            gridLine = Math.ceil(index) * gridWidth + referenceLine;   // 結果を切り上げ、大きい方のグリッド線を取得する
        }

        return gridLine;
    }

    GridManager.prototype.init = init;

    GridManager.prototype.goUpper = goUpper;
    GridManager.prototype.goLower = goLower;
    GridManager.prototype.goUL = goUL;
    GridManager.prototype.goLeft = goLeft;
    GridManager.prototype.goRight = goRight;
    GridManager.prototype.goLR = goLR;
    GridManager.prototype.goLRUL = goLRUL;
    GridManager.prototype.goUpperLeft = goUpperLeft;
    GridManager.prototype.goUpperRight = goUpperRight;
    GridManager.prototype.goLowerLeft = goLowerLeft;
    GridManager.prototype.goLowerRight = goLowerRight;

    GridManager.prototype.isMove = isMove;
    GridManager.prototype.nearGrid = nearGrid;
    GridManager.prototype.nextGrid = nextGrid;

})();

/**
 * 要素の移動を管理します。
 * キーボード及びポインティングデバイスによる移動をサポートします。
 */
com.yscjp.jsapp.element.JSMove = function () {};
(function() {

    var JSEvent = com.yscjp.jsapp.JSEvent;
    var Log = com.yscjp.jsapp.util.Log;
    var HtmlPoint = com.yscjp.jsapp.tools.HtmlPoint;
    var ArgChk = com.yscjp.jsapp.util.ArgChk;
    var GridManager = com.yscjp.jsapp.element.GridManager;
    var HtmlPosition = com.yscjp.jsapp.tools.HtmlPosition;
    var JSHtmlUtil = com.yscjp.jsapp.util.JSHtmlUtil;

    var beforeX = 0;
    var beforeY = 0;
    var moveList = new Object();
    var observerList = new Array();
    var currentMoveValue = null;
    var htmlPosition = new HtmlPosition();

    /**
     * このクラスが管理する要素が、選択及び移動したときに通知するオブザーバーを追加します。
     * observerは、selected(id), released(id)及びmoved(id)メソッドを実装している必要が有ります。
     * 引数の id は、移動または選択されれた要素のIDです。
     */
    function addObserver(observer) {

        ArgChk.typeOf(observer.selected, "function");
        ArgChk.typeOf(observer.released, "function");
        ArgChk.typeOf(observer.moved, "function");

        observerList[observerList.length] = observer;
    }

    function noticeSelected(id) {

        var returnValue = true;

        for (var i = 0; i < observerList.length; i++) {

            var result = observerList[i].selected(id);
            if (false == result) {

                returnValue = false;
            }
        }

        return returnValue;
    }

    function noticeReleased(id) {

        var returnValue = true;

        for (var i = 0; i < observerList.length; i++) {

            var result = observerList[i].released(id);
            if (false == result) {

                returnValue = false;
            }
        }

        return returnValue;
    }

    function noticeMoved(id) {

        var returnValue = true;

        for (var i = 0; i < observerList.length; i++) {

            var result = observerList[i].moved(id);
            if (false == result) {

                returnValue = false;
            }
        }

        return returnValue;
    }

    /**
     * このクラスに移動を管理させたい要素を追加します。
     */
    function add(id, gridWidthX, gridWidthY, referencePoint, isAdjustment, gridManager, isKeyMove) {

        ArgChk.typeOf(id, "string");
        var _gridWidthX = ArgChk.typeOf(gridWidthX, "number", 0);
        var _gridWidthY = ArgChk.typeOf(gridWidthY, "number", 0);
        var _referencePoint = ArgChk.instanceOf(referencePoint, HtmlPoint, null);
        var _isAdjustment = ArgChk.typeOf(isAdjustment, "boolean", false);
        var _gridManager = ArgChk.instanceOf(gridManager, GridManager, null);
        var _isKeyMove = ArgChk.typeOf(isKeyMove, "boolean", true);

        var element = document.getElementById(id);
        if (!element) {

            Log.warning('No element of the id. ' + id);
            return;
        }

        if (!_referencePoint) {

            /*
             * 基準点が指定されていない場合は、その要素の左上が基準点
             */
            _referencePoint = new HtmlPoint(element.offsetLeft, element.offsetTop);
        }

        if (!_gridManager) {

            /*
             * グリッドマネージャが指定されていない場合は、デフォルトのグリッドマネージャ
             */
            _gridManager = new GridManager();
            _gridManager.init(gridWidthX, gridWidthY, 0, 0, _referencePoint);
        }

        var moveValue = moveList[id];
        if (!moveValue) {

            moveValue = new Object();
        }

        moveValue.gridWidthX = _gridWidthX;
        moveValue.gridWidthY = _gridWidthY;
        moveValue.referencePoint = _referencePoint;
        moveValue.gridManager = _gridManager;
        moveValue.element = element;
        moveValue.id = id;
        moveValue.isKeyMove = _isKeyMove;
        moveValue.mouseDownNow = false;

        moveList[id] = moveValue;

        if (_referencePoint && _isAdjustment) {

            /*
             * 基準点が指定されており、アジャスト指示がある場合は、
             * その場所に要素の左上を合わせて移動させる。
             */
            HtmlPosition.moveTo(element, _referencePoint.y, _referencePoint.x);

        }

        JSEvent.addEventListener(element, "mousedown", mdown, false);
        JSEvent.addEventListener(element, "mousemove", mmoveElement, false);
        JSEvent.addEventListener(element, "mouseout", mout, false);

        if (!_isKeyMove) {

            /*
             * キーボードによる移動の指示が無い場合、ここまで。
             */
            return;
        }

        /* キーボードイベントを処理するテキストエリアを追加する。 */
        var focusCtrl = document.createElement('textarea');
        focusCtrl = document.body.appendChild(focusCtrl);

        if (com.yscjp.jsapp.util.UA.isFixedSupport()) {

            focusCtrl.style.position = 'fixed';

        } else {

            focusCtrl.style.position = 'absolute';
        }

        focusCtrl.style.top = '-100px';
        focusCtrl.style.left = '-100px';
        focusCtrl.style.width = '20px';
        focusCtrl.style.height = '20px';
        focusCtrl.style.border = '0px';
        focusCtrl.id = id + '_ctrl';
        moveValue.focusCtrl = focusCtrl;

        JSEvent.addEventListener(focusCtrl, "keydown", kdown, false);
        JSEvent.addEventListener(focusCtrl, "focus", focusHdl, false);
        JSEvent.addEventListener(focusCtrl, "blur", blurHdl, false);
    }

    function mmoveElement(e) {

        e = JSEvent.createEvent(e);

//        Log.debug("---- mmoveElement (" + e.currentTarget.id + ")----");

        if (!isRange(e)) {

            return;
        }

        e.currentTarget.style.cursor = 'move';
    }

    function mout(e) {

        e = JSEvent.createEvent(e);
        e.currentTarget.style.cursor = '';
    }

    function isRange(e) {

        var element = e.currentTarget;
        var resizeRangeV = Math.ceil(element.offsetHeight / 4);
        var resizeRangeH = Math.ceil(element.offsetWidth / 4);
        resizeRangeV = Math.min(resizeRangeV, 8);   // TODO このサイズをResizeと共有する方法
        resizeRangeH = Math.min(resizeRangeH, 8);

        if (e.offsetX < resizeRangeH || element.offsetWidth - resizeRangeH < e.offsetX) {

            return false;
        }

        if (e.offsetY < resizeRangeV || element.offsetHeight - resizeRangeV < e.offsetY) {

            return false;
        }

        return true;
    }

    function mdown(e) {

        e = JSEvent.createEvent(e);

//        Log.debug("---- mdown (" + e.currentTarget.id + ")----");

        if (!isRange(e)) {

            return;
        }

        beforeX = e.clientX;
        beforeY = e.clientY;
//        selectedElement = e.currentTarget;
        currentMoveValue = moveList[e.currentTarget.id];
        htmlPosition.setElement(currentMoveValue.element);
        htmlPosition.updatePosition();

        JSEvent.addEventListener(document, "mousemove", mmove, false);
        JSEvent.addEventListener(document, "mouseup", mup, false);

        if (currentMoveValue.isKeyMove) {

            JSHtmlUtil.addClassValue(currentMoveValue.element, "com_yscjp_htmlclass_selected");
            currentMoveValue.element.style.zIndex = '1';
            noticeSelected(currentMoveValue.id);
        }

        currentMoveValue.mouseDownNow = true;
    }

    function mmove(e) {

        if (null == currentMoveValue) {

            return;
        }

        e = JSEvent.createEvent(e);

        var moveX = e.clientX - beforeX;
        var moveY = e.clientY - beforeY;

        Log.debug("---- mmove (beforeX:" + beforeX + " clientX:" + e.clientX + " beforeY:" + beforeY + " clientY:"  + e.clientY + ")----");

        var leftTop = htmlPosition.getNWOnClient();
        Log.debug("---- mmove (leftTop[" + leftTop + "])----");
        var newPoint = null;

        if (0 < moveX) {

            if (0 >= moveY) {

                /*
                 * 右上に移動したので、右上のポイントを基点とする
                 */
                var beforeNE = htmlPosition.getNEOnClient();
                var currentNE = new HtmlPoint(beforeNE.x + moveX, beforeNE.y + moveY);

                /*
                 * X軸で＋方向
                 * Y軸で－方向
                 */
                currentNE = currentMoveValue.gridManager.goUpperRight(currentNE, beforeNE);

                newPoint = new HtmlPoint(currentNE.x - currentMoveValue.element.offsetWidth, currentNE.y);

            } else {

                /*
                 * 右下に移動したので、右下のポイントを基点とする
                 */
                var beforeSE = htmlPosition.getSEOnClient();
                var currentSE = new HtmlPoint(beforeSE.x + moveX, beforeSE.y + moveY);

                /*
                 * X軸で＋方向
                 * Y軸で＋方向
                 */
                currentSE = currentMoveValue.gridManager.goLowerRight(currentSE, beforeSE);

                newPoint = new HtmlPoint(currentSE.x - currentMoveValue.element.offsetWidth,
                                         currentSE.y - currentMoveValue.element.offsetHeight);

            }

        } else {

            if (0 >= moveY) {

                /*
                 * 左上に移動したので、左上のポイントを基点とする
                 */
                var beforeNW = htmlPosition.getNWOnClient();
                var currentNW = new HtmlPoint(beforeNW.x + moveX, beforeNW.y + moveY);

                /*
                 * X軸で－方向
                 * Y軸で－方向
                 */
                currentNW = currentMoveValue.gridManager.goUpperLeft(currentNW, beforeNW);

                newPoint = new HtmlPoint(currentNW.x, currentNW.y);

            } else {

                /*
                 * 左下に移動したので、左下のポイントを基点とする
                 */
                var beforeSW = htmlPosition.getSWOnClient();
                var currentSW = new HtmlPoint(beforeSW.x + moveX, beforeSW.y + moveY);

                /*
                 * X軸で－方向
                 * Y軸で＋方向
                 */
                currentSW = currentMoveValue.gridManager.goLowerLeft(currentSW, beforeSW);
                newPoint = new HtmlPoint(currentSW.x, currentSW.y - currentMoveValue.element.offsetHeight);

            }

        }

        if (newPoint.x != leftTop.x || newPoint.y != leftTop.y) {

            Log.debug("---- mmove (newPoint[" + newPoint + "])----");
        }

        htmlPosition.moveOffset(newPoint.y - leftTop.y, newPoint.x - leftTop.x);
        if (!noticeMoved(currentMoveValue.id)) {

            //htmlPosition.moveOffset((newPoint.y - leftTop.y) * -1, (newPoint.x - ) * -1);
            htmlPosition.moveTo(leftTop.y, leftTop.x);

        } else {

            beforeX += (newPoint.x - leftTop.x);
            beforeY += (newPoint.y - leftTop.y);
            htmlPosition.updatePosition();
        }
    }

    function mup(e) {

        if (null == currentMoveValue) {

            return;
        }

        e = JSEvent.createEvent(e);

        Log.debug("---- mup (" + e.currentTarget.id + ")----");

        if (currentMoveValue.focusCtrl) {

            currentMoveValue.focusCtrl.focus();
        }

        currentMoveValue.mouseDownNow = false;
        currentMoveValue = null;
        beforeX = 0;
        beforeY = 0;

        JSEvent.removeEventListener(document, "mousemove", mmove, false);
        JSEvent.removeEventListener(document, "mouseup", mup, false);
    }

    function kdown(e) {

        e = JSEvent.createEvent(e);

        Log.debug("---- kdown (" + e.currentTarget.id + ")----");
        var targetId = e.currentTarget.id.replace('_ctrl', "");
        var direction;

        switch (e.keyIdentifier) {
        case 37 : // 左
            direction = "W";
            break;
        case 38 : // 上
            direction = "N";
            break;
        case 39 : // 右
            direction = "E";
            break;
        case 40 : // 下
            direction = "S";
            break;
        default :
            return;
        }

        goNext(targetId, direction);
    }

    function focusHdl(e) {

        e = JSEvent.createEvent(e);

        var targetId = e.currentTarget.id.replace('_ctrl', "");
        var moveValue = moveList[targetId];
        JSHtmlUtil.addClassValue(moveValue.element, "com_yscjp_htmlclass_selected");
        moveValue.element.style.zIndex = '1';
        noticeSelected(targetId);
    }

    function blurHdl(e) {

        e = JSEvent.createEvent(e);

        var targetId = e.currentTarget.id.replace('_ctrl', "");
        var moveValue = moveList[targetId];
        if (moveValue.mouseDownNow) {
            /*
             * マウス操作中なので、選択の解除は行わない。
             */
            return;
        }
        JSHtmlUtil.removeClassValue(moveValue.element, "com_yscjp_htmlclass_selected");
        moveValue.element.style.zIndex = '';
        noticeReleased(targetId);
    }

    function goNext(id, direction) {

        var moveValue = moveList[id];
        htmlPosition.setElement(moveValue.element);
        htmlPosition.updatePosition();
        var top = htmlPosition.getTopOnClient();
        var left = htmlPosition.getLeftOnClient();
        var current = new HtmlPoint(left, top);
        var newPoint = moveValue.gridManager.nextGrid(current, direction);
        htmlPosition.moveOffset(newPoint.y - current.y, newPoint.x - current.x);
        htmlPosition.updatePosition();
        noticeMoved(id);
    }

    com.yscjp.jsapp.element.JSMove.addObserver = addObserver;
    com.yscjp.jsapp.element.JSMove.add = add;
    com.yscjp.jsapp.element.JSMove.goNext = goNext;

})();

com.yscjp.jsapp.element.ResizeManager = function (isTopResize,
                                                  isRightResize,
                                                  isBottomResize,
                                                  isLeftResize) {

    var ArgChk = com.yscjp.jsapp.util.ArgChk;

    this._isTopResize = ArgChk.typeOf(isTopResize, "boolean", true);
    this._isRightResize = ArgChk.typeOf(isRightResize, "boolean", true);
    this._isBottomResize = ArgChk.typeOf(isBottomResize, "boolean", true);
    this._isLeftResize = ArgChk.typeOf(isLeftResize, "boolean", true);
};
(function () {

    var ResizeManager = com.yscjp.jsapp.element.ResizeManager;

    ResizeManager.prototype.isTopResize = function () {

        return this._isTopResize;
    };

    ResizeManager.prototype.isRightResize = function () {

        return this._isRightResize;
    };

    ResizeManager.prototype.isBottomResize = function () {

        return this._isBottomResize;
    };

    ResizeManager.prototype.isLeftResize = function () {

        return this._isLeftResize;
    };

})();


com.yscjp.jsapp.element.JSResize = function () {};
(function () {

    var JSEvent = com.yscjp.jsapp.JSEvent;
    var Log = com.yscjp.jsapp.util.Log;
    var HtmlPoint = com.yscjp.jsapp.tools.HtmlPoint;
    var ArgChk = com.yscjp.jsapp.util.ArgChk;
    var GridManager = com.yscjp.jsapp.element.GridManager;
    var HtmlPosition = com.yscjp.jsapp.tools.HtmlPosition;
    var JSHtmlUtil = com.yscjp.jsapp.util.JSHtmlUtil;
    var ResizeManager = com.yscjp.jsapp.element.ResizeManager;

    var beforeX = 0;
    var beforeY = 0;
    var resizeList = new Object();
    var observerList = new Array();
    var currentResizeValue = null;
    var htmlPosition = new HtmlPosition();

    /**
     * このクラスが管理する要素が、サイズ変更したときに通知するオブザーバーを追加します。
     * observerは、resized(id)メソッドを実装している必要が有ります。
     * 引数の id は、移動または選択されれた要素のIDです。
     */
    function addObserver(observer) {

        ArgChk.typeOf(observer.resized, "function");

        observerList[observerList.length] = observer;
    }

    function noticeResized(id) {

        var returnValue = true;

        for (var i = 0; i < observerList.length; i++) {

            var result = observerList[i].resized(id);
            if (false == result) {

                returnValue = false;
            }
        }

        return returnValue;
    }

    /**
     * このクラスに移動を管理させたい要素を追加します。
     */
    function add(id,
                 gridWidthX,
                 gridWidthY,
                 referencePoint,
                 gridManager,
                 resizeManager) {

        ArgChk.typeOf(id, "string");
        var _gridWidthX = ArgChk.typeOf(gridWidthX, "number", 0);
        var _gridWidthY = ArgChk.typeOf(gridWidthY, "number", 0);
        var _referencePoint = ArgChk.instanceOf(referencePoint, HtmlPoint, null);
        var _gridManager = ArgChk.instanceOf(gridManager, GridManager, null);
        var _resizeManager = ArgChk.instanceOf(resizeManager, ResizeManager, null);

        var element = document.getElementById(id);
        if (!element) {

            Log.warning('No element of the id. ' + id);
            return;
        }

        if (!_referencePoint) {

            /*
             * 基準点が指定されていない場合は、その要素の左上が基準点
             */
            _referencePoint = new HtmlPoint(element.offsetLeft, element.offsetTop);
        }

        if (!_gridManager) {

            /*
             * グリッドマネージャが指定されていない場合は、デフォルトのグリッドマネージャ
             */
            _gridManager = new GridManager();
            _gridManager.init(gridWidthX, gridWidthY, 0, 0, _referencePoint);
        }

        if (!_resizeManager) {

            /*
             * リサイズマネージャが指定されていない場合は、デフォルトのリサイズマネージャ
             */
            _resizeManager = new ResizeManager(true, true, true, true);
        }

        var resizeValue = resizeList[id];
        if (!resizeValue) {

            resizeValue = new Object();
        }

        resizeValue.gridWidthX = _gridWidthX;
        resizeValue.gridWidthY = _gridWidthY;
        resizeValue.referencePoint = _referencePoint;
        resizeValue.gridManager = _gridManager;
        resizeValue.element = element;
        resizeValue.resizeManager = _resizeManager;
        resizeValue.id = id;
        resizeValue.mouseDownNow = false;

        resizeList[id] = resizeValue;

        JSEvent.addEventListener(element, "mousedown", mdown, false);
        JSEvent.addEventListener(element, "mousemove", mmoveElement, false);
        JSEvent.addEventListener(element, "mouseout", mout, false);
    }

    var cursorStyleTable = {"N"  : "n-resize",
                            "NE" : "ne-resize",
                            "E"  : "e-resize",
                            "SE" : "se-resize",
                            "S"  : "s-resize",
                            "SW" : "sw-resize",
                            "W"  : "w-resize",
                            "NW" : "nw-resize"
    };

    function mmoveElement(e) {

        e = JSEvent.createEvent(e);

        //Log.debug("---- mmoveElement (" + e.currentTarget.id + ")----");

        var cursorStyle = cursorStyleTable[getDirection(e)];
        if (!cursorStyle) {
            //cursorStyle = "";
            return;
        }
        e.currentTarget.style.cursor = cursorStyle;
    }

    function mout(e) {

        e = JSEvent.createEvent(e);
        e.currentTarget.style.cursor = '';
    }


    /**
     * 要素上のマウス位置がどの場所に該当するかを返します。
     * <pre>
     * +-----+---------------+-----+
     * | (a) |      (b)      | (c) |
     * +-----+---------------+-----+
     * |     |               |     |
     * |     |               |     |
     * | (d) |      (e)      | (f) |
     * |     |               |     |
     * |     |               |     |
     * +-----+---------------+-----+
     * | (g) |      (h)      | (i) |
     * +-----+---------------+-----+
     * </pre>
     * <p>上記のそれぞれの箇所で、返る値は以下のとおりです。
     * <ul>
     * <ui><p><tt>(a) → "NW"</tt>
     * <ui><p><tt>(b) → "N"</tt>
     * <ui><p><tt>(c) → "NE"</tt>
     * <ui><p><tt>(d) → "W"</tt>
     * <ui><p><tt>(e) → ""（空文字）</tt>
     * <ui><p><tt>(f) → "E"</tt>
     * <ui><p><tt>(g) → "SW"</tt>
     * <ui><p><tt>(h) → "S"</tt>
     * <ui><p><tt>(i) → "SE"</tt>
     * </ul>
     * <p>但し、その方向へのリサイズ指示が無い場合は、""（空文字）または異なる値を返します。
     * <p>左方向（→）へのリサイズ指示があり下方向（↓）のリサイズ指示が無い場合、(f)では"E"を返しますが、
     * (h)では""を返し(i)では"E"を返します。
     */
    function getDirection(e) {

        var element = e.currentTarget;
        var resizeRangeV = Math.ceil(element.offsetHeight / 4);
        var resizeRangeH = Math.ceil(element.offsetWidth / 4);
        resizeRangeV = Math.min(resizeRangeV, 8);
        resizeRangeH = Math.min(resizeRangeH, 8);

        var resizeValue = resizeList[e.currentTarget.id];
        var resizeManager = resizeValue.resizeManager;

        var direction = "";

        if (e.offsetX < resizeRangeH) {

            /* (a), (d), (g) */

            if (resizeManager.isLeftResize()) {

                direction = "W";

                if (e.offsetY < resizeRangeV) {

                    /*  (a) */

                    if (resizeManager.isTopResize()) {

                        direction = "NW";
                    }

                } else if (element.offsetHeight - resizeRangeV < e.offsetY) {

                    /*  (g) */

                    if (resizeManager.isBottomResize()) {

                        direction = "SW";
                    }
                }
            }

        } else if (element.offsetWidth - resizeRangeH < e.offsetX) {

            /* (c), (f), (i) */

            if (resizeManager.isRightResize()) {

                direction = "E";

                if (e.offsetY < resizeRangeV) {

                    /* (c) */

                    if (resizeManager.isTopResize()) {

                        direction = "NE";
                    }

                } else if (element.offsetHeight - resizeRangeV < e.offsetY) {

                    /* (i) */

                    if (resizeManager.isBottomResize()) {

                        direction = "SE";
                    }
                }
            }
        }

        if (!direction) {

            if (e.offsetY < resizeRangeV) {

                /* (a), (b), (c) */

                if (resizeManager.isTopResize()) {

                    direction = "N";
                }

            } else  if (element.offsetHeight - resizeRangeV < e.offsetY) {

                /* (g), (h), (i) */

                if (resizeManager.isBottomResize()) {

                    direction = "S";
                }
            }
        }

        return direction;
    }

    function mdown(e) {

        e = JSEvent.createEvent(e);

        Log.debug("---- mdown (" + e.currentTarget.id + ")----");

        var direction = getDirection(e);
        if (!direction) {

            return;
        }

        beforeX = e.clientX;
        beforeY = e.clientY;
//        selectedElement = e.currentTarget;
        currentResizeValue = resizeList[e.currentTarget.id];
        htmlPosition.setElement(currentResizeValue.element);
        htmlPosition.updatePosition();

        switch (direction) {
        case "NW" :
        case "N" :
        case "W" :

            JSEvent.addEventListener(document, "mousemove", mmoveNW, false);
            currentResizeValue.downFunc = mmoveNW;
            break;

        case "SE" :
        case "S" :
        case "E" :

            JSEvent.addEventListener(document, "mousemove", mmoveSE, false);
            currentResizeValue.downFunc = mmoveSE;
            break;

        case "NE" :

            JSEvent.addEventListener(document, "mousemove", mmoveNE, false);
            currentResizeValue.downFunc = mmoveNE;
            break;

        case "SW" :

            JSEvent.addEventListener(document, "mousemove", mmoveSW, false);
            currentResizeValue.downFunc = mmoveSW;
            break;
        }

        JSEvent.addEventListener(document, "mouseup", mup, false);

        currentResizeValue.mouseDownNow = true;
        currentResizeValue.direction = direction;
    }


    /**
     * 要素の左上を基点としてリサイズ処理を行うマウス移動イベントのハンドラです。
     * <p>マウスの位置が、NW, N, Wの時にマウスダウンイベントが発生したケースで登録されることを、
     * 前提としています。
     * <pre>
     * +(a)     +(b)
     *
     *     NW+-------------+
     *       |             |
     * +(c)  |  +(d)       |
     *       |             |
     *       |             |
     *       +-------------+
     * </pre>
     * <p>NWが、(a)(b)(c)(d)の何れかに移動した場合に、要素のサイズを変更します。
     */
    function mmoveNW(e) {

        if (null == currentResizeValue) {

            return;
        }

        e = JSEvent.createEvent(e);

        Log.debug("---- mmoveNW ----");

        var currentDirection = "SE";

        /*
         * マウスが移動した距離を、現在位置から前回の位置をマイナスして取得する。
         * (a)へ移動した場合は、moveXがマイナス、moveYもマイナス。
         * (b)へ移動した場合は、moveXがプラス、moveYがマイナス。
         * (c)へ移動した場合は、moveXがマイナス、moveYがプラス。
         * (d)へ移動した場合は、moveXがプラス、moveYもプラス。
         */
        var moveX = e.clientX - beforeX;
        var moveY = e.clientY - beforeY;

        var currentWidth = htmlPosition.getWidth();
        var currentHeight = htmlPosition.getHeight();

        var elementReference = htmlPosition.getNWOnClient();
        var currentReference = new HtmlPoint(elementReference.x + moveX, elementReference.y + moveY);
        var newPosition = currentResizeValue.gridManager.goLRUL(currentReference, elementReference);

        /*
         * 新しいサイズを算出する。
         */
        newWidth = currentWidth - (newPosition.x - elementReference.x);
        newHeight = currentHeight - (newPosition.y - elementReference.y);

//        Log.debug("---- mmove (newWidth:" + newWidth + "/newHeight:" + newHeight + " currentDirection" + currentDirection + ")----");
        if ("N" == currentResizeValue.direction || "S" == currentResizeValue.direction) {

            /*
             * 上下の変化のみ有効で左右の変化は無視する。
             */
            newWidth = currentWidth;
        }

        if ("E" == currentResizeValue.direction || "W" == currentResizeValue.direction) {

            /*
             * 左右の変化のみ有効で上下の変化は無視する。
             */
            newHeight = currentHeight;
        }

        if (newWidth != currentWidth || newHeight != currentHeight) {

            //Log.info("---- mmove (newWidth:" + newWidth + "/newHeight:" + newHeight + " currentDirection:" + currentDirection + "currentResizeValue.direction:" + currentResizeValue.direction + ")----");

            HtmlPosition.resizeBase(currentResizeValue.element, currentDirection, newHeight, newWidth);
            if (false == noticeResized(currentResizeValue.id)) {

                Log.debug("noticeResized returnd false");

                /*
                 * 通知の結果、falseが返った場合は元のサイズに戻す。
                 */
                HtmlPosition.resizeBase(currentResizeValue.element, currentDirection, currentHeight, currentWidth);

            } else {

                beforeX -= (newWidth - currentWidth);
                beforeY -= (newHeight - currentHeight);
                htmlPosition.updatePosition();
            }
        }
    }


    /**
     * 要素の右下を基点としてリサイズ処理を行うマウス移動イベントのハンドラです。
     * マウスの位置が、E, SE, Sの時にマウスダウンイベントが発生したケースで登録されることを、
     * 前提としています。
     * <pre>
     * +-------------+
     * |             |
     * |             |
     * |        +(a) | +(b)
     * |             |
     * +-------------+SE
     *
     *          +(c)   +(d)
     * </pre>
     * <p>SEが、(a)(b)(c)(d)の何れかに移動した場合に、要素のサイズを変更します。
     */
    function mmoveSE(e) {

        if (null == currentResizeValue) {

            Log.debug("---- mmoveSE(null == currentResizeValue) ----");
            return;
        }

        e = JSEvent.createEvent(e);

        Log.debug("---- mmoveSE ----");

        var currentDirection = "NW";

        /*
         * マウスが移動した距離を、現在位置から前回の位置をマイナスして取得する。
         * (a)へ移動した場合は、moveXがマイナス、moveYもマイナス。
         * (b)へ移動した場合は、moveXがプラス、moveYがマイナス。
         * (c)へ移動した場合は、moveXがマイナス、moveYがプラス。
         * (d)へ移動した場合は、moveXがプラス、moveYもプラス。
         */
        var moveX = e.clientX - beforeX;
        var moveY = e.clientY - beforeY;

        var currentWidth = htmlPosition.getWidth();
        var currentHeight = htmlPosition.getHeight();

        var elementReference = htmlPosition.getSEOnClient();
        var currentReference = new HtmlPoint(elementReference.x + moveX, elementReference.y + moveY);
        var newPosition = currentResizeValue.gridManager.goLRUL(currentReference, elementReference);

        /*
         * 新しいサイズを算出する。
         */
        newWidth = currentWidth + (newPosition.x - elementReference.x);
        newHeight = currentHeight + (newPosition.y - elementReference.y);

//        Log.debug("---- mmove (newWidth:" + newWidth + "/newHeight:" + newHeight + " currentDirection" + currentDirection + ")----");
        if ("N" == currentResizeValue.direction || "S" == currentResizeValue.direction) {

            /*
             * 上下の変化のみ有効で左右の変化は無視する。
             */
            newWidth = currentWidth;
        }

        if ("E" == currentResizeValue.direction || "W" == currentResizeValue.direction) {

            /*
             * 左右の変化のみ有効で上下の変化は無視する。
             */
            newHeight = currentHeight;
        }

        if (newWidth != currentWidth || newHeight != currentHeight) {

            //Log.info("---- mmove (newWidth:" + newWidth + "/newHeight:" + newHeight + " currentDirection:" + currentDirection + "currentResizeValue.direction:" + currentResizeValue.direction + ")----");

            HtmlPosition.resizeBase(currentResizeValue.element, currentDirection, newHeight, newWidth);
            if (false == noticeResized(currentResizeValue.id)) {

                /*
                 * 通知の結果、falseが返った場合は元のサイズに戻す。
                 */
                HtmlPosition.resizeBase(currentResizeValue.element, currentDirection, currentHeight, currentWidth);

            } else {

                beforeX += (newWidth - currentWidth);
                beforeY += (newHeight - currentHeight);
                htmlPosition.updatePosition();
            }
        }
    }


    /**
     * 要素の右上を基点としてリサイズ処理を行うマウス移動イベントのハンドラです。
     * マウスの位置が、NEの時にマウスダウンイベントが発生したケースで登録されることを、
     * 前提としています。
     * <pre>
     *          +(a)   +(b)
     *
     * +-------------+NE
     * |             |
     * |        +(c) | +(d)
     * |             |
     * |             |
     * +-------------+
     * </pre>
     * <p>NWが、(a)(b)(c)(d)の何れかに移動した場合に、要素のサイズを変更します。
     */
    function mmoveNE(e) {

        if (null == currentResizeValue) {

            return;
        }

        e = JSEvent.createEvent(e);

        Log.debug("---- mmoveNE ----");

        var currentDirection = "SW";

        /*
         * マウスが移動した距離を、現在位置から前回の位置をマイナスして取得する。
         * (a)へ移動した場合は、moveXがマイナス、moveYもマイナス。
         * (b)へ移動した場合は、moveXがプラス、moveYがマイナス。
         * (c)へ移動した場合は、moveXがマイナス、moveYがプラス。
         * (d)へ移動した場合は、moveXがプラス、moveYもプラス。
         */
        var moveX = e.clientX - beforeX;
        var moveY = e.clientY - beforeY;

        var currentWidth = htmlPosition.getWidth();
        var currentHeight = htmlPosition.getHeight();

        var elementReference = htmlPosition.getNEOnClient();
        var currentReference = new HtmlPoint(elementReference.x + moveX, elementReference.y + moveY);
        var newPosition = currentResizeValue.gridManager.goLRUL(currentReference, elementReference);

        /*
         * 新しいサイズを算出する。
         */
        newWidth = currentWidth + (newPosition.x - elementReference.x);
        newHeight = currentHeight - (newPosition.y - elementReference.y);

        if (newWidth != currentWidth || newHeight != currentHeight) {

            //Log.info("---- mmove (newWidth:" + newWidth + "/newHeight:" + newHeight + " currentDirection:" + currentDirection + "currentResizeValue.direction:" + currentResizeValue.direction + ")----");

            HtmlPosition.resizeBase(currentResizeValue.element, currentDirection, newHeight, newWidth);
            if (false == noticeResized(currentResizeValue.id)) {

                /*
                 * 通知の結果、falseが返った場合は元のサイズに戻す。
                 */
                HtmlPosition.resizeBase(currentResizeValue.element, currentDirection, currentHeight, currentWidth);

            } else {

                beforeX += (newWidth - currentWidth);
                beforeY -= (newHeight - currentHeight);
                htmlPosition.updatePosition();
            }
        }
    }


    /**
     * 要素の左下を基点としてリサイズ処理を行うマウス移動イベントのハンドラです。
     * マウスの位置が、SWの時にマウスダウンイベントが発生したケースで登録されることを、
     * 前提としています。
     * <pre>
     *       +-------------+
     *       |             |
     *       |             |
     * +(a)  |  +(b)       |
     *       |             |
     *     SW+-------------+
     *
     * +(c)     +(d)
     * </pre>
     * <p>SWが、(a)(b)(c)(d)の何れかに移動した場合に、要素のサイズを変更します。
     */
    function mmoveSW(e) {

        if (null == currentResizeValue) {

            return;
        }

        e = JSEvent.createEvent(e);

        Log.debug("---- mmoveSW ----");

        var currentDirection = "NE";

        /*
         * マウスが移動した距離を、現在位置から前回の位置をマイナスして取得する。
         * (a)へ移動した場合は、moveXがマイナス、moveYもマイナス。
         * (b)へ移動した場合は、moveXがプラス、moveYがマイナス。
         * (c)へ移動した場合は、moveXがマイナス、moveYがプラス。
         * (d)へ移動した場合は、moveXがプラス、moveYもプラス。
         */
        var moveX = e.clientX - beforeX;
        var moveY = e.clientY - beforeY;

        var currentWidth = htmlPosition.getWidth();
        var currentHeight = htmlPosition.getHeight();

        var elementReference = htmlPosition.getSWOnClient();
        var currentReference = new HtmlPoint(elementReference.x + moveX, elementReference.y + moveY);
        var newPosition = currentResizeValue.gridManager.goLRUL(currentReference, elementReference);

        /*
         * 新しいサイズを算出する。
         */
        newWidth = currentWidth - (newPosition.x - elementReference.x);
        newHeight = currentHeight + (newPosition.y - elementReference.y);

        if (newWidth != currentWidth || newHeight != currentHeight) {

            HtmlPosition.resizeBase(currentResizeValue.element, currentDirection, newHeight, newWidth);
            if (false == noticeResized(currentResizeValue.id)) {

                /*
                 * 通知の結果、falseが返った場合は元のサイズに戻す。
                 */
                HtmlPosition.resizeBase(currentResizeValue.element, currentDirection, currentHeight, currentWidth);

            } else {

                beforeX -= (newWidth - currentWidth);
                beforeY += (newHeight - currentHeight);
                htmlPosition.updatePosition();
            }
        }
    }

    function mup(e) {

        if (null == currentResizeValue) {

            Log.debug("---- mup (null == currentResizeValue)----");
            return;
        }

        e = JSEvent.createEvent(e);

        Log.debug("---- mup (" + currentResizeValue.direction + ")----");

        currentResizeValue.mouseDownNow = false;

        switch (currentResizeValue.direction) {
        case "NW" :
        case "N" :
        case "W" :

            JSEvent.removeEventListener(document, "mousemove", currentResizeValue.downFunc, false);
            break;

        case "SE" :
        case "S" :
        case "E" :

            JSEvent.removeEventListener(document, "mousemove", currentResizeValue.downFunc, false);
            break;

        case "NE" :

            JSEvent.removeEventListener(document, "mousemove", currentResizeValue.downFunc, false);
            break;

        case "SW" :

            JSEvent.removeEventListener(document, "mousemove", currentResizeValue.downFunc, false);
            break;

        default :

            Log.debug("---- mup (currentResizeValue.direction is illegal:"+currentResizeValue.direction+")----");

        }

        JSEvent.removeEventListener(document, "mouseup", mup, false);
        currentResizeValue = null;
        beforeX = 0;
        beforeY = 0;
    }

    com.yscjp.jsapp.element.JSResize.addObserver = addObserver;
    com.yscjp.jsapp.element.JSResize.add = add;

})();

com.yscjp.jsapp.element.TabManager = function () {

    this.index = 0;
    this.itemList = new Array();
};
(function () {

    var TabManager = com.yscjp.jsapp.element.TabManager;
    var JSEvent = com.yscjp.jsapp.JSEvent;
    var Log = com.yscjp.jsapp.util.Log;
    var ArgChk = com.yscjp.jsapp.util.ArgChk;
    var JSHtmlUtil = com.yscjp.jsapp.util.JSHtmlUtil;

    function addTab(id, selectedHdl, releaseHdl, isDefault, targetDocument) {

        id = ArgChk.notNull(id);
        selectedHdl = ArgChk.notNull(selectedHdl, null);
        releaseHdl = ArgChk.notNull(releaseHdl, null);
        isDefault = ArgChk.notNull(isDefault, false);
        targetDocument = ArgChk.notNull(targetDocument, document);
        tabElement = ArgChk.notNull(targetDocument.getElementById(id));

        var tabItem = new Object();
        tabItem.id = id;
        tabItem.selectedHdl = selectedHdl;
        tabItem.releaseHdl = releaseHdl;
        tabItem.targetDocument = targetDocument;
        tabItem.tabElement = tabElement;

        if (isDefault) {

            JSHtmlUtil.addClassValue(tabItem.tabElement, "com_yscjp_htmlclass_selectTab");

        } else {

            JSHtmlUtil.addClassValue(tabItem.tabElement, "com_yscjp_htmlclass_nonSelectTab");
        }

        this.itemList.push(tabItem);

        var thisAction = this.action;
        var __this__ = this;
        JSEvent.addEventListener(tabElement, "click", function () {
            thisAction.apply(__this__, arguments);
        }, false);

        this.index ++;
    }

    function action(e) {

        Log.debug(e.currentTarget.id);
        var currentId = e.currentTarget.id;
        this.changeTab(currentId);
    }

    function changeTab(id) {

        var selectedHdl = null;

        for (var i = 0; i < this.itemList.length; i++) {

            var tabItem = this.itemList[i];

            if (id == tabItem.id) {

                JSHtmlUtil.removeClassValue(tabItem.tabElement, "com_yscjp_htmlclass_nonSelectTab");
                JSHtmlUtil.addClassValue(tabItem.tabElement, "com_yscjp_htmlclass_selectTab");
                selectedHdl = tabItem.selectedHdl;

            } else {

                JSHtmlUtil.removeClassValue(tabItem.tabElement, "com_yscjp_htmlclass_selectTab");
                JSHtmlUtil.addClassValue(tabItem.tabElement, "com_yscjp_htmlclass_nonSelectTab");

                if (tabItem.releaseHdl) {

                    tabItem.releaseHdl.apply(null, arguments);
                }
            }
        }

        if (selectedHdl) {

            selectedHdl.apply(null, arguments);
        }
    }

    TabManager.prototype.addTab = addTab;
    TabManager.prototype.action = action;
    TabManager.prototype.changeTab = changeTab;

})();
